function A = aloc_idid_L1tri(tau,x)
        
% A = aloc_idid_L1tri(tau,x)
%             calculates a local contribution to the system matrix
%             that corresponds to the bilinear form:
%              a(v,u) = tau * \int_{K^e} v(x)*u(x) dx
%             L1tri stands for the linear Lagrange (i.e. nodal) triangular
%             elements.
%
%             The nodal degrees of freedom are nodal values at the following
%             nodes:
%               x1 (reference is (0,0)), x2 (reference is (1,0)),
%               x3 (reference is (0,1)).

x1 = x(:,1);
x2 = x(:,2);
x3 = x(:,3);

R = [x2-x1 x3-x1];

wGP = [1/6 1/6 1/6];
xGP = [[0;1/2] [1/2;0] [1/2;1/2]];

A = zeros(3,3);

for i=1:3 
  B = [1-xGP(1,i)-xGP(2,i),xGP(1,i),xGP(2,i)];  % linear shape functions
  A = A + wGP(i)*B'*B;
end

A = tau*A*abs(det(R));
