function [x,elemNodes,nodalID,edges,elemEdges,edgeID,elemID] = discret_L1tri(xl,xr,xb,xt,h)

% [x,e,nodalID,edges,elemEdges,edgeID,elemID] = discret_L1tri(xl,xr,xb,xt,h)
%          discretizes a rectangle [xl,xr] x [xb,xt]
%          into reverse-clockwise-oriented triangles
%          with a discretization step h.
%          x is a list of nodal coordinates,
%          elemNodes is a list of nodes per element,
%          nodalID is a list of nodes along the boundary,
%          edges is a list of edges,
%          elemEdges is a list of oriented edges per element,
%            the positive orientation is e1=x1->x2, e2 = x2->x3,
%            e3 = x3->x1.
%          The latter two outputs are used for refinement and
%            for Crouzeix-Raviart elements.
%          edgeID is a list of edges along the boundary.


x1 = xl:h:xr;
x2 = xb:h:xt;

nx1 = max(size(x1));
nx2 = max(size(x2));

x = [];
elemNodes = [];
nodalID = [];
edges = [];
elemEdges = [];
edgeID = [];
elemID = [];

index = 1;
elindex = 1;

edgeindex = 0;

% seznam uzlu, hran a elementu
for i=1:nx1
    for j=1:nx2
        x = [x [x1(i);x2(j)]];
                
        if i<nx1 & j<nx2
            elemNodes = [ elemNodes , [index;index+nx2+1;index+1] , [index;index+nx2;index+nx2+1] ];
            elindex = elindex+2;
            
            edges = [ edges , [index;index+nx2] , [index;index+nx2+1] , [index;index+1] ];
            
            if j==1
                edgeID = [edgeID;edgeindex+1];
                elemID = [elemID;elindex-1;elindex];
            end
            if i==1
                edgeID = [edgeID;edgeindex+3];
                elemID = [elemID;elindex-1;elindex];
            end
            
            if i<nx1-1
                elemEdges = [ elemEdges , [edgeindex+2;-(edgeindex+4);-(edgeindex+3)] , [edgeindex+1;edgeindex+3*nx2+1;-(edgeindex+2)] ];
            else
                elemEdges = [ elemEdges , [edgeindex+2;-(edgeindex+4);-(edgeindex+3)] , [edgeindex+1;edgeindex+3*(nx2-j)+1+j;-(edgeindex+2)] ];
            end
            
            edgeindex = edgeindex+3;
            
        elseif j==nx2 & i<nx1
            edges = [ edges , [index;index+nx2] ];
            edgeID = [edgeID;edgeindex+1];
            edgeindex = edgeindex+1;
            elemID = [elemID;elindex-1;elindex];
            
        elseif i==nx1 & j<nx2
            edges = [ edges , [index;index+1] ];
            edgeID = [edgeID;edgeindex+1];
            edgeindex = edgeindex+1;
            elemID = [elemID;elindex-1;elindex];
            
        end
        
        if i==1 | i==nx1 | j==1 | j==nx2
            nodalID = [nodalID;index];
        end
               
        index = index + 1;
    end
end
